﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;
using System.Windows.Forms;
using System.Drawing;
using System.Resources;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides general information about the computer. 
    /// </summary>
    public static class Computer
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_OperatingSystem");
        private static ManagementObjectSearcher searcher2 = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_ComputerSystem");

        /// <summary>
        /// Returns the current computer name.
        /// </summary>
        public static string ComputerName
        {
            get
            {
                return SystemInformation.ComputerName;
            }
        }

        /// <summary>
        /// Returns the name of this device.
        /// </summary>
        public static string DeviceName
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("SystemProductName").ToString();
            }
        }

        /// <summary>
        /// Returns the manufacturer of this device.
        /// </summary>
        public static string DeviceManufacturer
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("SystemManufacturer").ToString();
            }
        }

        /// <summary>
        /// Returns the family of this device.
        /// </summary>
        public static string DeviceFamily
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("SystemFamily").ToString();
            }
        }

        /// <summary>
        /// Returns the elapsed time since the last system start.
        /// </summary>
        public static TimeSpan ElapsedTimeSinceStart
        {
            get
            {
                TimeSpan time = new TimeSpan((int)((Environment.TickCount / 60000) / 60), (int)(Environment.TickCount / 60000), (int)(Environment.TickCount / 1000));
                return time;
            }
        }

        /// <summary>
        /// Returns the description of this computer.
        /// </summary>
        public static string Description
        {
            get
            {
                string descript = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    descript = queryObj["Description"].ToString();
                }
                return descript;
            }
        }

        /// <summary>
        /// Returns the path of the current desktop background.
        /// </summary>
        public static string CurrentWallpaper
        {
            get
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("Wallpaper").ToString();
            }
        }

        /// <summary>
        /// Returns the path of the current screensaver.
        /// </summary>
        public static string CurrentScreensaver
        {
            get
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("SCRNSAVE.EXE").ToString();
            }
        }

        /// <summary>
        /// Returns the workgroup of the computer
        /// </summary>
        public static string Workgroup
        {
            get
            {
                string wg = "";
                foreach (ManagementObject queryObj in searcher2.Get())
                {
                    wg = queryObj["Workgroup"].ToString();
                }
                return wg;
            }
        }
    }
}
