﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Win32;
using System.Diagnostics;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize desktop settings.
    /// </summary>
    public static class Desktop
    {

        /// <summary>
        /// Shows/hides informations in the lower right corner of the desktop.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool PaintDesktopVersion
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("PaintDesktopVersion").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("PaintDesktopVersion", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("PaintDesktopVersion", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Shows/hides the system tray items.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool DisplayTrayItems
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Policies\\Explorer", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoTrayItemsDisplay").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Policies\\Explorer", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Policies", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("Explorer", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                }
                    
                if (value)
                { regKey.SetValue("NoTrayItemsDisplay", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("NoTrayItemsDisplay", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables the animation when windows are being minimized/maximized.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool WindowAnimation
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop\\WindowMetrics", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("MinAnimate").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop\\WindowMetrics", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("MinAnimate", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("MinAnimate", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables AeroShake.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool AeroShake
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Explorer", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoWindowMinimizingShortcuts").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows\\Explorer", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Policies\\Microsoft\\Windows", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("Explorer", RegistryKeyPermissionCheck.ReadWriteSubTree);
                }
         
                if (value)
                { regKey.SetValue("NoWindowMinimizingShortcuts", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("NoWindowMinimizingShortcuts", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Enables/disables Snap.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool Snap
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("WindowArrangementActive").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Control Panel\\Desktop", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("WindowArrangementActive", "1", RegistryValueKind.String); }
                else
                { regKey.SetValue("WindowArrangementActive", "0", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Changes the time, how long the user has to wait for the AeroPeek preview.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static int AeroPeekDelayTime
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Advanced", true);
                if (regKey == null)
                { return 1000; }

                try
                {
                    ans = int.Parse(regKey.GetValue("DesktopLivePreviewHoverTime").ToString());
                    return ans;
                }
                catch { return 1000; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadWriteSubTree);
                regKey.SetValue("DesktopLivePreviewHoverTime", value, RegistryValueKind.DWord);
            }
        }

        /// <summary>
        /// Enables/disables notification balloons.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool NotificationBalloons
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EnableBalloonTips").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("EnableBalloonTips", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EnableBalloonTips", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Shows/hides the shortcut to the libraries on the desktop.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool LibrariesOnDesktop
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\HideDesktopIcons\\NewStartPanel", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("{031E4825-7B94-4dc3-B131-E946B44C8DD5}").ToString()); }
                catch { return false; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\HideDesktopIcons", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("HideDesktopIcons", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .CreateSubKey("NewStartPanel", RegistryKeyPermissionCheck.ReadWriteSubTree);
                }
                else
                {
                    regKey = regKey.OpenSubKey("NewStartPanel", RegistryKeyPermissionCheck.ReadWriteSubTree);
                }
                    
                if (value)
                { regKey.SetValue("{031E4825-7B94-4dc3-B131-E946B44C8DD5}", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("{031E4825-7B94-4dc3-B131-E946B44C8DD5}", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Shows/hides the recycle bin shortcut on 'This PC'.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool RecycleBinOnComputer
        {
            get
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\MyComputer\\NameSpace\\{645FF040-5081-101B-9F08-00AA002F954E}", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\MyComputer\\NameSpace", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("MyComputer", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("NameSpace", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                }
            
                if (value)
                { regKey.CreateSubKey("{645FF040-5081-101B-9F08-00AA002F954E}"); }
                else
                { regKey.DeleteSubKey("{645FF040-5081-101B-9F08-00AA002F954E}"); }
            }
        }

        /// <summary>
        /// Enables/disables the preview of the desktop shown if the mouse is in the lower-right-corner.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool DesktopPreview
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("DisablePreviewDesktop").ToString()); }
                catch { return true; }
                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("DisablePreviewDesktop", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("DisablePreviewDesktop", 1, RegistryValueKind.DWord); }
            }
        }

        #region Taskbar

        private const string VistaStartMenuCaption = "Start";
        private static IntPtr vistaStartMenuWnd = IntPtr.Zero;

        /// <summary>
        /// Returns the window handle of the Vista start menu orb.
        /// </summary>
        /// <param name="taskBarWnd">windo handle of taskbar</param>
        /// <returns>window handle of start menu</returns>
        private static IntPtr GetVistaStartMenuWnd(IntPtr taskBarWnd)
        {
            int procId = 0;
            Win32Api.GetWindowThreadProcessId(taskBarWnd, ref procId);

            Process p = Process.GetProcessById(procId);
            if (p != null)
            {
                foreach (ProcessThread t in p.Threads)
                {
                    Win32Api.EnumThreadWindows(t.Id, MyEnumThreadWindowsProc, IntPtr.Zero);
                }
            }
            return vistaStartMenuWnd;
        }

        /// <summary>
        /// Callback method that is called from 'EnumThreadWindows' in 'GetVistaStartMenuWnd'.
        /// </summary>
        /// <param name="hWnd">window handle</param>
        /// <param name="lParam">parameter</param>
        /// <returns>true to continue enumeration, false to stop it</returns>
        private static bool MyEnumThreadWindowsProc(IntPtr hWnd, IntPtr lParam)
        {
            StringBuilder buffer = new StringBuilder(256);
            if (Win32Api.GetWindowText(hWnd, buffer, buffer.Capacity) > 0)
                if (buffer.ToString() == VistaStartMenuCaption)
                {
                    vistaStartMenuWnd = hWnd;
                    return false;
                }
            return true;
        }

        /// <summary>
        /// Sets, whether the taskbar is visible.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public static bool TaskbarVisible
        {
            set
            {
                IntPtr taskBarWnd = Win32Api.FindWindow("Shell_TrayWnd", null);

                IntPtr startWnd = Win32Api.FindWindowEx(taskBarWnd, IntPtr.Zero, "Button", "Start");

                if (startWnd == IntPtr.Zero)
                {
                    startWnd = Win32Api.FindWindowEx(IntPtr.Zero, IntPtr.Zero, (IntPtr)0xC017, "Start");
                }

                if (startWnd == IntPtr.Zero)
                {
                    startWnd = Win32Api.FindWindow("Button", null);

                    if (startWnd == IntPtr.Zero)
                    { startWnd = GetVistaStartMenuWnd(taskBarWnd); }
                }

                int winstate = 0;
                if (value) { winstate = Win32Api.SW_SHOW; }
                else { winstate = Win32Api.SW_HIDE; }
                Win32Api.ShowWindow(taskBarWnd, winstate);
                Win32Api.ShowWindow(startWnd, winstate);
            }
        }
        #endregion

        /// <summary>
        /// Returns the current desktop background or changes it.
        /// </summary>
        /// <value></value>
        /// <returns></returns>
        /// <remarks></remarks>
        public static string Wallpaper
        {
            get
            { return Info.Computer.CurrentWallpaper; }
            set
            { Win32Api.SystemParametersInfo(Win32Api.SPI_SETDESKWALLPAPER, 0, value, Win32Api.SPIF_UPDATEINIFILE | Win32Api.SPIF_SENDWININICHANGE); }
        }

        /// <summary>
        /// Enables/disables the usage of check boxes in the upper left corner of files for selection.
        /// </summary>
        public static bool CheckBoxSelection
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("AutoCheckSelect").ToString()); }
                catch { return false; }

                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("AutoCheckSelect", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("AutoCheckSelect", 0, RegistryValueKind.DWord); }
            }
        }
    }
}
